import axios from 'axios';
import { parse, stringify } from 'qs';

const instance = axios.create();
const token = 'bearer ' + localStorage.getItem("token");

let defaultHeaders = {
    token,
  };
  
  const codeMessage = {
    200: '服务器成功返回请求的数据。',
    201: '新建或修改数据成功。',
    202: '一个请求已经进入后台排队（异步任务）。',
    204: '删除数据成功。',
    400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
    401: '用户没有权限（用户名或密码错误）。',
    403: '用户得到授权，但是访问是被禁止的。',
    404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
    406: '请求的格式不可得。',
    410: '请求的资源被永久删除，且不会再得到的。',
    422: '当创建一个对象时，发生一个验证错误。',
    500: '服务器发生错误，请检查服务器。',
    502: '网关错误。',
    503: '服务不可用，服务器暂时过载或维护。',
    504: '网关超时。',
  };

export const createAPI = (baseURL) => {
    return (conf = {}) => {
        let opts = conf.opts || {};
        let body = conf.body || {};
        let urlParams = '';


        let headers = { ...defaultHeaders, ...opts.headers };
        if (conf.queryType === 'query') {
            urlParams = conf.body ? "?" + stringify(conf.body) : '';
        }

        return instance(Object.assign({}, {
            url: conf.url + urlParams,
            baseURL: baseURL,
            method: conf.method,
            data: body,
            headers,
        }, conf.opts))
            .then(function (response) {
                if (!response || response.data === false) { console.log('response出错！', response); return false; }
                // 保存到本地
                if (response && response.data && response.data.access_token) {
                    defaultHeaders = {
                        token: 'bearer ' + response.data.access_token,
                    }
                    localStorage.setItem("token", response.data.access_token);
                }

                if (response.data && response.data.success || response.data && (response.data.success === undefined && response.data.error === undefined)) {
                    return response.data;
                } else {
                    message.error(response.data && (response.data.mesg || response.data.message) || '操作出现错误！')
                    return response.data;
                }

            })
            .catch(error => {
                if (error.response) {
                    const { status, statusText, url } = error.response;
                    const errortext = codeMessage[status] || statusText;
                    // notification.error({
                    //   key: status,
                    //   message: `请求错误 ${status}`,
                    //   description: errortext,
                    // });
                    if (status === 401) {
                        // @HACK
                        /* eslint-disable no-underscore-dangle */

                        history.push('/user/login');
                        message.error('登录过期或用户名密码错误！')
                        return false;
                    } else if (status === 400 || status > 401) {
                        history.push(`/exception/${status}`);
                    }
                }
                return false;
            });
    };
}

export const convertRESTAPI = (url, opts) => {
    if (!opts) return url;

    const pathKeys = Object.keys(opts);

    pathKeys.forEach((key) => {
        const r = new RegExp('(:' + key + '|{' + key + '})', 'g');
        url = url.replace(r, opts[key]);
    });
    return url;
}


